#!/usr/bin/perl -w

#===============================================================================
#
#    Created: 18/05/2009 09:56:29
#    Copyright (C) 2009, University of Liverpool.
#    Author: Kevin Ashelford.
#
#    Contact details:
#    Email:   k.ashelford@liv.ac.uk
#    Address: School of Biological Sciences, University of Liverpool, 
#             Biosciences Building, Crown Street, Liverpool, UK. L69 7ZB
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#===============================================================================

use strict;
use warnings;
use Getopt::Std;
use File::Basename;

#///////////////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////////////

my $_program = basename($0);

sub usage {
	
die <<MESSAGE;
		
usage: $_program
	[-i SOLiD reads infile (csfasta.ma)]
	[-r reference sequence (fasta)]
	[-o coverage outfile (csv)] 
   or: $_program -h (for help)
	
MESSAGE
}

#///////////////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////////////

if (!@ARGV) {
	warn "\nERROR: too few arguments\n\n";
	usage;
	}
main(@ARGV);

#///////////////////////////////////////////////////////////////////////////////

# MAIN

sub main {

	if ($ARGV[0] eq "-h") {
		usage;
		}
	
	# Get arguments.
	#===========================================================================
	
	my %options;
	getopts ("i:o:r:", \%options);
	my $infile = $options{i} or usage;
	my $reffile = $options{r} or usage;
	my $outfile = $options{o} or usage;
	
	#===========================================================================
	
	
	# Open infile.
	#===========================================================================
	
	my $refLength = 0;
	
	print ">>> Reading $reffile...\n";
	open (REF, $reffile) or die "\nERROR: Could not open $reffile - $!";
	while (<REF>) {
	
		chomp;
		next if (/^>/);
		
		if (/^([ACGTN\s]+)/i) {
			my $line = uc($1);
			$line =~ s/\s//;
			$refLength += length($line);
			}
		
		}
	close (REF) or die "\nERROR: Could not close $reffile - $!";
	
	# Initialise array for storing coverage data. 
	my @covArray;
	for (my $i = 0; $i < $refLength; $i++) {
		$covArray[$i] = 0;
		}
	
	#===========================================================================
	
	
	my $header;
	my $sequence;
	
	print ">>> Reading $infile...\n";
	open(IN, "$infile") or die "\nERROR: Could not open $infile: $!";
	while (<IN>) {
		
		chomp;
		
		if (/^>(\S+)$/) {
			$header = $1;
			}
		
		elsif (/^(T\d+)$/) {
			$sequence = $1;
			processRead($header, $sequence, \@covArray);
			}
		
		}
	
	close(IN) or die "\nERROR: Could not close $infile: $!";
	
	#===========================================================================
	
	
	# Write outfile.
	#===========================================================================
	
	print ">>> Writing $outfile...\n";
	open(OUT, ">$outfile") or die "\nERROR: Could not create $outfile: $!";
	foreach my $cov (@covArray) {
		print OUT "$cov\n";
		}
	close(OUT) or die "\nERROR: Could not close $outfile: $!";
	
	#===========================================================================
	
	print ">>> Program finished.\n\n";	

	} # End of main.

#///////////////////////////////////////////////////////////////////////////////

# METHODS

sub processRead {
	
	my $header = shift;
	my $sequence = shift;
	my $covArray = shift;
	
		
	# Parse header - at this stage assuming only uniquely mapping reads.
	# e.g. 919_550_1784_F3,6_-2644649.3
	$header =~ m/^(\d+\_\d+\_\d+\_[FR]3),(\d+)\_(-{0,1})(\d+)\.(\d+)$/ or
		die "Unexpected header - $header.";
	
	my $start = $4;
	my $strand = $3;
	my $n = length($sequence);
	my $refLength = @$covArray;
	

	
	
	
	if ($strand eq "-") {

		for (my $i = $start+1; $i > $start-$n+2; $i--) {
			$covArray->[$i]++;	
			}
		
		}
	else {
		
		for (my $i = $start+1; $i < $start+$n; $i++) {
			$covArray->[$i]++;	
			}
		
		
		}
	

	} # End of method.
	
#///////////////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////////////
1;

