#!/usr/bin/perl -w

#===============================================================================
#
#    Created: 15/05/2009 18:18:35
#    Copyright (C) 2009, University of Liverpool.
#    Author: Kevin Ashelford.
#
#    Contact details:
#    Email:   k.ashelford@liv.ac.uk
#    Address: School of Biological Sciences, University of Liverpool, 
#             Biosciences Building, Crown Street, Liverpool, UK. L69 7ZB
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#===============================================================================

use strict;
use warnings;
use Getopt::Std;
use File::Basename;
use IO::File;

#///////////////////////////////////////////////////////////////////////////////

my $splitByChromosome = sub {
	
	my $header = shift;
	my $sequence = shift;
	my $outdir = shift;
	
	my $chr;
	if ($header =~ m/^\d+_\d+_\d+_[FR]\d,(\d+)[_\.]/) {
		
		$chr = "chr$1";
		
		}
	elsif ($header =~ m/^\d+_\d+_\d+_[FR]\d,(\w+)[_\.]/) {
		
		$chr = $1;
		
		}
	else {
		die "Could not match '$header' - $!";
		}
	
	my $FILEHANDLE = getFilehandle($chr, $outdir);
	
	print $FILEHANDLE ">$header\n";
	print $FILEHANDLE "$sequence\n";
	
	}; #End of method.

#///////////////////////////////////////////////////////////////////////////////

my $splitByStrand = sub {
	
	my $header = shift;
	my $sequence = shift;
	my $outdir = shift;
	
	$header =~ m/^\d+_\d+_\d+_[FR]\d,\d+_(-{0,1})/ or 
		die "Could not match '$header' - $!";
	
	my $strand = "plus";
	if ($1 eq "-") {
		$strand = "minus";	
		}
	
	my $FILEHANDLE = getFilehandle($strand, $outdir);
	
	print $FILEHANDLE ">$header\n";
	print $FILEHANDLE "$sequence\n";
	
	};

#///////////////////////////////////////////////////////////////////////////////

my $splitByMismatch = sub {
	
	my $header = shift;
	my $sequence = shift;
	my $outdir = shift;
	
	$header =~ m/^\d+_\d+_\d+_[FR]\d,\d+_-{0,1}\d+\.(\d+)$/ 
		or die "Could not match '$header' - $!";
	
	my $mismatch = $1;
	
	my $FILEHANDLE = getFilehandle($mismatch, $outdir);
	
	print $FILEHANDLE ">$header\n";
	print $FILEHANDLE "$sequence\n";
	
	};

#///////////////////////////////////////////////////////////////////////////////

my %_FILEHANDLES;
my %_splitByParameter = (
	
	chromosome => $splitByChromosome,
	strand => $splitByStrand,
	mismatch => $splitByMismatch
	
	);

#///////////////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////////////

my $_program = basename($0);

sub usage {
	
die <<MESSAGE;
		
usage: $_program
	[-i SOLiD read infile (csfasta)]
	[-o outfile directory]
	[-p parameter to split by (options: chromosome, strand, mismatch)]
   or: $_program -h (for help)
	
MESSAGE
}

#///////////////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////////////

if (!@ARGV) {
	warn "\nERROR: too few arguments\n\n";
	usage;
	}
main(@ARGV);

#///////////////////////////////////////////////////////////////////////////////

# MAIN

sub main {

	if ($ARGV[0] eq "-h") {
		usage;
		}
	
	# Get arguments.
	#===========================================================================
	
	my %options;
	getopts ("i:o:p:", \%options);
	my $infile = $options{i} or usage;
	my $outdir = $options{o} or usage;
	my $parameter = $options{p} or usage;
	
	die "Parameter '$parameter' is not recognised.\n" 
		if (!exists $_splitByParameter{$parameter});
	
	mkdir ($outdir) or die "\nERROR: could not create new $outdir.\n";
	
	#===========================================================================
	
	
	# Open infile.
	#===========================================================================
	
	my $header;
	my $sequence;
	
	print ">>> Reading $infile...\n";
	open(IN, "$infile") or die "\nERROR: Could not open $infile: $!";
	while (<IN>) {
		
		chomp;
		
		if (/^>(\S+)$/) {
			$header = $1;
			}
		
		elsif (/^(T\d+)$/) {
			$sequence = $1;
			$_splitByParameter{$parameter}($header, $sequence, $outdir);
			}
		
		}
	close(IN) or die "\nERROR: Could not close $infile: $!";
	
	#===========================================================================
	
	
	foreach my $chr (keys %_FILEHANDLES) {
		close $_FILEHANDLES{$chr} or
			die "\nERROR: Could not close filehandle for chr$chr - $!";
		}
	
	
	print ">>> Program finished.\n\n";	

	} # End of main.

#///////////////////////////////////////////////////////////////////////////////

# METHODS

sub getFilehandle {
	
	my $chr = shift;
	my $outdir = shift;
	
	return $_FILEHANDLES{$chr} if (exists $_FILEHANDLES{$chr});
	
	my $outfile = "$outdir/$chr.csfasta.ma";
	
	my $FILEHANDLE = IO::File->new;
	
	open ($FILEHANDLE, ">$outfile") or die "\nERROR: could not create ";
	$_FILEHANDLES{$chr} = $FILEHANDLE;
	
	return $FILEHANDLE;
	
	} # End of method.
	
#///////////////////////////////////////////////////////////////////////////////
#///////////////////////////////////////////////////////////////////////////////
1;

